/*
 * Copyright (c) 2024 NVIDIA CORPORATION AND AFFILIATES.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright notice, this list of
 *       conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 *     * Neither the name of the NVIDIA CORPORATION nor the names of its contributors may be used
 *       to endorse or promote products derived from this software without specific prior written
 *       permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NVIDIA CORPORATION BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TOR (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _PSP_GW_PARAMS_H_
#define _PSP_GW_PARAMS_H_

#include <string>
#include <vector>
#include <doca_error.h>

#define MAX_FILE_NAME (255) /* Maximum file name length */

struct psp_gw_app_config;

/**
 * @brief Parses command-line arguments to the application.
 * During processing of arguments, both DPDK and the application
 * may remove arguments from argv, and argc will reflect the
 * new size.
 *
 * @argc [in/out]: The number of args passed to main()
 * @argv [in/out]: The args passed to main
 * @app_config [out]: The configuration of the application
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t psp_gw_argp_exec(int &argc, char *argv[], psp_gw_app_config *app_config);

/**
 * @brief Parses the configuration JSON file to the application.
 *
 * @app_config [in/out]: The configuration of the application
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t psp_gw_parse_config_file(psp_gw_app_config *app_config);

#endif /* _PSP_GW_PARAMS_H_ */
